@@ -156,17 +156,37 @@ sysprof_capture_reader_discover_end_time (SysprofCaptureReader *self)
     {
       gint64 end_time = frame.time;
 
-      if (frame.type == SYSPROF_CAPTURE_FRAME_MARK)
+      switch (frame.type)
         {
-          const SysprofCaptureMark *mark = NULL;
+        case SYSPROF_CAPTURE_FRAME_MARK: {
+            const SysprofCaptureMark *mark = NULL;
 
-          if ((mark = sysprof_capture_reader_read_mark (self)))
-            end_time = frame.time + MAX (0, mark->duration);
+            if ((mark = sysprof_capture_reader_read_mark (self)))
+              end_time = frame.time + MAX (0, mark->duration);
+          }
+          break;
+
+        case SYSPROF_CAPTURE_FRAME_ALLOCATION:
+        case SYSPROF_CAPTURE_FRAME_CTRSET:
+        case SYSPROF_CAPTURE_FRAME_EXIT:
+        case SYSPROF_CAPTURE_FRAME_FORK:
+        case SYSPROF_CAPTURE_FRAME_LOG:
+        case SYSPROF_CAPTURE_FRAME_PROCESS:
+        case SYSPROF_CAPTURE_FRAME_SAMPLE:
+        case SYSPROF_CAPTURE_FRAME_TIMESTAMP:
+          if (end_time > self->end_time)
+            self->end_time = end_time;
+          break;
+
+        case SYSPROF_CAPTURE_FRAME_MAP:
+        case SYSPROF_CAPTURE_FRAME_JITMAP:
+        case SYSPROF_CAPTURE_FRAME_CTRDEF:
+        case SYSPROF_CAPTURE_FRAME_METADATA:
+        case SYSPROF_CAPTURE_FRAME_FILE_CHUNK:
+        default:
+          break;
         }
 
-      if (end_time > self->end_time)
-        self->end_time = end_time;
-
       if (!sysprof_capture_reader_skip (self))
         break;
     }
@@ -336,6 +356,9 @@ sysprof_capture_reader_ensure_space_for (SysprofCaptureReader *self,
   g_assert (self->pos <= self->len);
   g_assert (len > 0);
 
+  /* Ensure alignment of length to read */
+  len = (len + SYSPROF_CAPTURE_ALIGN - 1) & ~(SYSPROF_CAPTURE_ALIGN - 1);
+
   if ((self->len - self->pos) < len)
     {
       gssize r;
@@ -419,10 +442,14 @@ sysprof_capture_reader_peek_frame (SysprofCaptureReader *self,
 
   sysprof_capture_reader_bswap_frame (self, frame);
 
+  /* In case the capture did not update the end_time during normal usage,
+   * we can update our cached known end_time based on the greatest frame
+   * we come across.
+   */
   if (frame->time > self->end_time)
     self->end_time = frame->time;
 
-  return TRUE;
+  return frame->type > 0 && frame->type < SYSPROF_CAPTURE_FRAME_LAST;
 }
 
 gboolean
@@ -439,7 +466,7 @@ sysprof_capture_reader_peek_type (SysprofCaptureReader    *self,
 
   *type = frame.type;
 
-  return TRUE;
+  return frame.type > 0 && frame.type < SYSPROF_CAPTURE_FRAME_LAST;
 }
 
 static const SysprofCaptureFrame *
@@ -467,6 +494,9 @@ sysprof_capture_reader_read_basic (SysprofCaptureReader    *self,
   if (frame->type != type)
     return NULL;
 
+  if (frame->len > (self->len - self->pos))
+    return NULL;
+
   self->pos += frame->len;
 
   return frame;
@@ -1352,3 +1382,52 @@ sysprof_capture_reader_find_file (SysprofCaptureReader *self,
 
   return NULL;
 }
+
+const SysprofCaptureAllocation *
+sysprof_capture_reader_read_allocation (SysprofCaptureReader *self)
+{
+  SysprofCaptureAllocation *ma;
+
+  g_assert (self != NULL);
+  g_assert ((self->pos % SYSPROF_CAPTURE_ALIGN) == 0);
+  g_assert (self->pos <= self->bufsz);
+
+  if (!sysprof_capture_reader_ensure_space_for (self, sizeof *ma))
+    return NULL;
+
+  ma = (SysprofCaptureAllocation *)(gpointer)&self->buf[self->pos];
+
+  sysprof_capture_reader_bswap_frame (self, &ma->frame);
+
+  if (ma->frame.type != SYSPROF_CAPTURE_FRAME_ALLOCATION)
+    return NULL;
+
+  if (ma->frame.len < sizeof *ma)
+    return NULL;
+
+  if (self->endian != G_BYTE_ORDER)
+    {
+      ma->n_addrs = GUINT16_SWAP_LE_BE (ma->n_addrs);
+      ma->alloc_size = GUINT64_SWAP_LE_BE (ma->alloc_size);
+      ma->alloc_addr = GUINT64_SWAP_LE_BE (ma->alloc_addr);
+      ma->tid = GUINT32_SWAP_LE_BE (ma->tid);
+    }
+
+  if (ma->frame.len < (sizeof *ma + (sizeof(SysprofCaptureAddress) * ma->n_addrs)))
+    return NULL;
+
+  if (!sysprof_capture_reader_ensure_space_for (self, ma->frame.len))
+    return NULL;
+
+  ma = (SysprofCaptureAllocation *)(gpointer)&self->buf[self->pos];
+
+  if (G_UNLIKELY (self->endian != G_BYTE_ORDER))
+    {
+      for (guint i = 0; i < ma->n_addrs; i++)
+        ma->addrs[i] = GUINT64_SWAP_LE_BE (ma->addrs[i]);
+    }
+
+  self->pos += ma->frame.len;
+
+  return ma;
+}
