@@ -78,6 +78,7 @@
 #define DEFAULT_BUFFER_SIZE (_sysprof_getpagesize() * 64L)
 #define INVALID_ADDRESS     (G_GUINT64_CONSTANT(0))
 #define MAX_COUNTERS        ((1 << 24) - 1)
+#define MAX_UNWIND_DEPTH    64
 
 typedef struct
 {
@@ -487,7 +488,17 @@ sysprof_capture_writer_new_from_fd (int   fd,
   self->next_counter_id = 1;
 
   now = g_date_time_new_now_local ();
+
+#if GLIB_CHECK_VERSION(2, 62, 0)
   nowstr = g_date_time_format_iso8601 (now);
+#else
+  {
+    GTimeVal tv;
+
+    g_date_time_to_timeval (now, &tv);
+    nowstr = g_time_val_to_iso8601 (&tv);
+  }
+#endif
 
   header = sysprof_capture_writer_allocate (self, &header_len);
 
@@ -1510,3 +1521,128 @@ sysprof_capture_writer_set_flush_delay (SysprofCaptureWriter *self,
 
   g_source_attach (self->periodic_flush, main_context);
 }
+
+gboolean
+sysprof_capture_writer_add_allocation (SysprofCaptureWriter  *self,
+                                       gint64                 time,
+                                       gint                   cpu,
+                                       gint32                 pid,
+                                       gint32                 tid,
+                                       SysprofCaptureAddress  alloc_addr,
+                                       gint64                 alloc_size,
+                                       SysprofBacktraceFunc   backtrace_func,
+                                       gpointer               backtrace_data)
+{
+  SysprofCaptureAllocation *ev;
+  gsize len;
+  guint n_addrs;
+
+  g_assert (self != NULL);
+  g_assert (backtrace_func != NULL);
+
+  len = sizeof *ev + (MAX_UNWIND_DEPTH * sizeof (SysprofCaptureAddress));
+  ev = (SysprofCaptureAllocation *)sysprof_capture_writer_allocate (self, &len);
+  if (!ev)
+    return FALSE;
+
+  sysprof_capture_writer_frame_init (&ev->frame,
+                                     len,
+                                     cpu,
+                                     pid,
+                                     time,
+                                     SYSPROF_CAPTURE_FRAME_ALLOCATION);
+
+  ev->alloc_size = alloc_size;
+  ev->alloc_addr = alloc_addr;
+  ev->padding1 = 0;
+  ev->tid = tid;
+  ev->n_addrs = 0;
+
+  n_addrs = backtrace_func (ev->addrs, MAX_UNWIND_DEPTH, backtrace_data);
+
+  if (n_addrs <= MAX_UNWIND_DEPTH)
+    ev->n_addrs = n_addrs;
+
+  if (ev->n_addrs < MAX_UNWIND_DEPTH)
+    {
+      gsize diff = (sizeof (SysprofCaptureAddress) * (MAX_UNWIND_DEPTH - ev->n_addrs));
+
+      ev->frame.len -= diff;
+      self->pos -= diff;
+    }
+
+  self->stat.frame_count[SYSPROF_CAPTURE_FRAME_ALLOCATION]++;
+
+  return TRUE;
+}
+
+gboolean
+sysprof_capture_writer_add_allocation_copy (SysprofCaptureWriter        *self,
+                                            gint64                       time,
+                                            gint                         cpu,
+                                            gint32                       pid,
+                                            gint32                       tid,
+                                            SysprofCaptureAddress        alloc_addr,
+                                            gint64                       alloc_size,
+                                            const SysprofCaptureAddress *addrs,
+                                            guint                        n_addrs)
+{
+  SysprofCaptureAllocation *ev;
+  gsize len;
+
+  g_assert (self != NULL);
+
+  if (n_addrs > 0xFFF)
+    n_addrs = 0xFFF;
+
+  len = sizeof *ev + (n_addrs * sizeof (SysprofCaptureAddress));
+  ev = (SysprofCaptureAllocation *)sysprof_capture_writer_allocate (self, &len);
+  if (!ev)
+    return FALSE;
+
+  sysprof_capture_writer_frame_init (&ev->frame,
+                                     len,
+                                     cpu,
+                                     pid,
+                                     time,
+                                     SYSPROF_CAPTURE_FRAME_ALLOCATION);
+
+  ev->alloc_size = alloc_size;
+  ev->alloc_addr = alloc_addr;
+  ev->padding1 = 0;
+  ev->tid = tid;
+  ev->n_addrs = n_addrs;
+
+  memcpy (ev->addrs, addrs, sizeof (SysprofCaptureAddress) * n_addrs);
+
+  self->stat.frame_count[SYSPROF_CAPTURE_FRAME_ALLOCATION]++;
+
+  return TRUE;
+}
+
+gboolean
+_sysprof_capture_writer_add_raw (SysprofCaptureWriter      *self,
+                                 const SysprofCaptureFrame *fr)
+{
+  gpointer begin;
+  gsize len;
+
+  g_assert (self != NULL);
+  g_assert ((fr->len & 0x7) == 0);
+  g_assert (fr->type < SYSPROF_CAPTURE_FRAME_LAST);
+
+  len = fr->len;
+
+  if (!(begin = sysprof_capture_writer_allocate (self, &len)))
+    return FALSE;
+
+  g_assert (fr->len == len);
+  g_assert (fr->type < 16);
+
+  memcpy (begin, fr, fr->len);
+
+  if (fr->type < G_N_ELEMENTS (self->stat.frame_count))
+    self->stat.frame_count[fr->type]++;
+
+  return TRUE;
+}
