@@ -42,6 +42,8 @@
 #include "sysprof-diskstat-aid.h"
 #include "sysprof-logs-aid.h"
 #include "sysprof-marks-aid.h"
+#include "sysprof-memory-aid.h"
+#include "sysprof-memprof-aid.h"
 #include "sysprof-netdev-aid.h"
 #include "sysprof-rapl-aid.h"
 
@@ -237,10 +239,24 @@ sysprof_display_dup_title (SysprofDisplay *self)
 
   if (priv->reader != NULL)
     {
+      g_autoptr(GDateTime) dt = NULL;
       const gchar *filename;
+      const gchar *capture_time;
 
       if ((filename = sysprof_capture_reader_get_filename (priv->reader)))
         return g_path_get_basename (filename);
+
+      /* This recording is not yet on disk, but the time it was recorded
+       * is much more useful than "New Recording".
+       */
+      capture_time = sysprof_capture_reader_get_time (priv->reader);
+
+      if ((dt = g_date_time_new_from_iso8601 (capture_time, NULL)))
+        {
+          g_autofree gchar *formatted = g_date_time_format (dt, "%X");
+          /* translators: %s is replaced with locale specific time of recording */
+          return g_strdup_printf (_("Recording at %s"), formatted);
+        }
     }
 
   return g_strdup (_("New Recording"));
@@ -327,7 +343,11 @@ change_page_cb (GSimpleAction *action,
   if (g_variant_is_of_type (param, G_VARIANT_TYPE_STRING))
     {
       const gchar *str = g_variant_get_string (param, NULL);
+
       gtk_stack_set_visible_child_name (priv->pages, str);
+
+      if (g_str_equal (str, "details"))
+        sysprof_visualizers_frame_unselect_row (priv->visualizers);
     }
 }
 
@@ -604,7 +624,10 @@ sysprof_display_present_cb (GObject      *object,
   g_assert (G_IS_TASK (task));
 
   if (!sysprof_aid_present_finish (aid, result, &error))
-    g_warning ("Failed to present aid %s: %s", G_OBJECT_TYPE_NAME (aid), error->message);
+    {
+      if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED))
+        g_warning ("Failed to present aid %s: %s", G_OBJECT_TYPE_NAME (aid), error->message);
+    }
 
   n_active = g_task_get_task_data (task);
 
@@ -636,6 +659,8 @@ sysprof_display_present_async (SysprofDisplay       *self,
   g_ptr_array_add (aids, sysprof_diskstat_aid_new ());
   g_ptr_array_add (aids, sysprof_logs_aid_new ());
   g_ptr_array_add (aids, sysprof_marks_aid_new ());
+  g_ptr_array_add (aids, sysprof_memory_aid_new ());
+  g_ptr_array_add (aids, sysprof_memprof_aid_new ());
   g_ptr_array_add (aids, sysprof_netdev_aid_new ());
   g_ptr_array_add (aids, sysprof_rapl_aid_new ());
 
@@ -703,7 +728,8 @@ sysprof_display_scan_worker (GTask        *task,
 
   while (sysprof_capture_reader_peek_frame (reader, &frame))
     {
-      st.frame_count[frame.type]++;
+      if (frame.type < G_N_ELEMENTS (st.frame_count))
+        st.frame_count[frame.type]++;
 
       if (frame.type == SYSPROF_CAPTURE_FRAME_METADATA)
         {
@@ -1025,7 +1051,6 @@ sysprof_display_open (SysprofDisplay *self,
   SysprofDisplayPrivate *priv = sysprof_display_get_instance_private (self);
   g_autoptr(SysprofCaptureReader) reader = NULL;
   g_autoptr(GError) error = NULL;
-  g_autoptr(GTask) task = NULL;
   g_autofree gchar *path = NULL;
 
   g_return_if_fail (SYSPROF_IS_DISPLAY (self));
@@ -1033,10 +1058,19 @@ sysprof_display_open (SysprofDisplay *self,
   g_return_if_fail (g_file_is_native (file));
   g_return_if_fail (sysprof_display_is_empty (self));
 
-  g_set_object (&priv->file, file);
-
   path = g_file_get_path (file);
 
+  /* If the file is executable, just set the path to the binary
+   * in the profiler assistant.
+   */
+  if (g_file_test (path, G_FILE_TEST_IS_EXECUTABLE))
+    {
+      sysprof_profiler_assistant_set_executable (priv->assistant, path);
+      return;
+    }
+
+  g_set_object (&priv->file, file);
+
   if (!(reader = sysprof_capture_reader_new (path, &error)))
     g_warning ("Failed to open capture: %s", error->message);
   else
@@ -1194,3 +1228,23 @@ sysprof_display_save (SysprofDisplay *self)
   g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_TITLE]);
   gtk_native_dialog_destroy (GTK_NATIVE_DIALOG (native));
 }
+
+void
+_sysprof_display_reload_page (SysprofDisplay *self,
+                              SysprofPage    *page)
+{
+  SysprofDisplayPrivate *priv = sysprof_display_get_instance_private (self);
+  SysprofSelection *selection;
+
+  g_return_if_fail (SYSPROF_IS_DISPLAY (self));
+  g_return_if_fail (SYSPROF_IS_PAGE (page));
+  g_return_if_fail (priv->reader != NULL);
+
+  selection = sysprof_visualizers_frame_get_selection (priv->visualizers);
+
+  sysprof_page_load_async (page,
+                           priv->reader,
+                           selection,
+                           priv->filter,
+                           NULL, NULL, NULL);
+}
