@@ -41,6 +41,7 @@
 #include "sysprof-callgraph-aid.h"
 #include "sysprof-cpu-aid.h"
 #include "sysprof-memory-aid.h"
+#include "sysprof-memprof-aid.h"
 #include "sysprof-netdev-aid.h"
 #include "sysprof-proxy-aid.h"
 #include "sysprof-rapl-aid.h"
@@ -397,6 +398,7 @@ sysprof_profiler_assistant_class_init (SysprofProfilerAssistantClass *klass)
   g_type_ensure (SYSPROF_TYPE_DISKSTAT_SOURCE);
   g_type_ensure (SYSPROF_TYPE_ENVIRON_EDITOR);
   g_type_ensure (SYSPROF_TYPE_MEMORY_AID);
+  g_type_ensure (SYSPROF_TYPE_MEMPROF_AID);
   g_type_ensure (SYSPROF_TYPE_NETDEV_AID);
   g_type_ensure (SYSPROF_TYPE_PROXY_AID);
   g_type_ensure (SYSPROF_TYPE_RAPL_AID);
@@ -455,3 +457,22 @@ _sysprof_profiler_assistant_focus_record (SysprofProfilerAssistant *self)
 
   gtk_widget_grab_focus (GTK_WIDGET (self->record_button));
 }
+
+void
+sysprof_profiler_assistant_set_executable (SysprofProfilerAssistant *self,
+                                           const gchar              *path)
+{
+  g_return_if_fail (SYSPROF_IS_PROFILER_ASSISTANT (self));
+
+  if (path == NULL || path[0] == 0)
+    {
+      gtk_entry_set_text (GTK_ENTRY (self->command_line), "");
+      gtk_switch_set_active (self->launch_switch, FALSE);
+    }
+  else
+    {
+      gtk_entry_set_text (GTK_ENTRY (self->command_line), path);
+      gtk_switch_set_active (self->launch_switch, TRUE);
+      gtk_widget_grab_focus (GTK_WIDGET (self->command_line));
+    }
+}
