@@ -116,9 +116,9 @@ sysprof_callgraph_profile_finalize (GObject *object)
 
 static void
 sysprof_callgraph_profile_get_property (GObject    *object,
-                                   guint       prop_id,
-                                   GValue     *value,
-                                   GParamSpec *pspec)
+                                        guint       prop_id,
+                                        GValue     *value,
+                                        GParamSpec *pspec)
 {
   SysprofCallgraphProfile *self = SYSPROF_CALLGRAPH_PROFILE (object);
 
@@ -135,9 +135,9 @@ sysprof_callgraph_profile_get_property (GObject    *object,
 
 static void
 sysprof_callgraph_profile_set_property (GObject      *object,
-                                   guint         prop_id,
-                                   const GValue *value,
-                                   GParamSpec   *pspec)
+                                        guint         prop_id,
+                                        const GValue *value,
+                                        GParamSpec   *pspec)
 {
   SysprofCallgraphProfile *self = SYSPROF_CALLGRAPH_PROFILE (object);
 
@@ -180,7 +180,7 @@ sysprof_callgraph_profile_init (SysprofCallgraphProfile *self)
 
 static void
 sysprof_callgraph_profile_set_reader (SysprofProfile       *profile,
-                                 SysprofCaptureReader *reader)
+                                      SysprofCaptureReader *reader)
 {
   SysprofCallgraphProfile *self = (SysprofCallgraphProfile *)profile;
 
@@ -275,7 +275,6 @@ sysprof_callgraph_profile_generate_worker (GTask        *task,
   while (sysprof_capture_reader_peek_type (reader, &type))
     {
       const SysprofCaptureProcess *pr;
-      const gchar *cmdline;
 
       if (type != SYSPROF_CAPTURE_FRAME_PROCESS)
         {
@@ -287,10 +286,13 @@ sysprof_callgraph_profile_generate_worker (GTask        *task,
       if (NULL == (pr = sysprof_capture_reader_read_process (reader)))
         goto failure;
 
-      cmdline = g_strdup_printf ("[%s]", pr->cmdline);
-      g_hash_table_insert (cmdlines,
-                           GINT_TO_POINTER (pr->frame.pid),
-                           (gchar *)sysprof_callgraph_profile_intern_string (self, cmdline));
+      if (!g_hash_table_contains (cmdlines, GINT_TO_POINTER (pr->frame.pid)))
+        {
+          g_autofree gchar *cmdline = g_strdup_printf ("[%s]", pr->cmdline);
+          g_hash_table_insert (cmdlines,
+                               GINT_TO_POINTER (pr->frame.pid),
+                               (gchar *)sysprof_callgraph_profile_intern_string (self, cmdline));
+        }
     }
 
   if (g_task_return_error_if_cancelled (task))
@@ -337,6 +339,13 @@ sysprof_callgraph_profile_generate_worker (GTask        *task,
 
       cmdline = g_hash_table_lookup (cmdlines, GINT_TO_POINTER (sample->frame.pid));
 
+      if (cmdline == NULL)
+        {
+          gchar *pidstr = g_strdup_printf ("[Process %d]", sample->frame.pid);
+          g_hash_table_insert (cmdlines, GINT_TO_POINTER (sample->frame.pid), pidstr);
+          cmdline = pidstr;
+        }
+
 #if 0
       /* This assertion appears to hold true, but since we're taking in
        * untrusted data from capture files, it's not safe to assume. But in
@@ -373,6 +382,13 @@ sysprof_callgraph_profile_generate_worker (GTask        *task,
             }
           else
             {
+              /* In case we get plain backtraces that aren't coming from perf,
+               * we might never get a context switch into user-space. This ensures
+               * that we still get traces for things from backtrace().
+               */
+              if (last_context == SYSPROF_ADDRESS_CONTEXT_NONE)
+                last_context = SYSPROF_ADDRESS_CONTEXT_USER;
+
               for (guint j = 0; j < resolvers->len; j++)
                 {
                   SysprofSymbolResolver *resolver = g_ptr_array_index (resolvers, j);
@@ -411,9 +427,7 @@ sysprof_callgraph_profile_generate_worker (GTask        *task,
           g_array_index (resolved, SysprofAddress, len++) = POINTER_TO_U64 (name);
         }
 
-      if (cmdline != NULL)
-        g_array_index (resolved, guint64, len++) = POINTER_TO_U64 (cmdline);
-
+      g_array_index (resolved, guint64, len++) = POINTER_TO_U64 (cmdline);
       g_array_index (resolved, guint64, len++) = POINTER_TO_U64 ("[Everything]");
 
       stack_stash_add_trace (resolved_stash, (gpointer)resolved->data, len, 1);
@@ -446,10 +460,10 @@ generate_free (Generate *generate)
 }
 
 static void
-sysprof_callgraph_profile_generate (SysprofProfile           *profile,
-                               GCancellable        *cancellable,
-                               GAsyncReadyCallback  callback,
-                               gpointer             user_data)
+sysprof_callgraph_profile_generate (SysprofProfile      *profile,
+                                    GCancellable        *cancellable,
+                                    GAsyncReadyCallback  callback,
+                                    gpointer             user_data)
 {
   SysprofCallgraphProfile *self = (SysprofCallgraphProfile *)profile;
   Generate *gen;
@@ -469,9 +483,9 @@ sysprof_callgraph_profile_generate (SysprofProfile           *profile,
 }
 
 static gboolean
-sysprof_callgraph_profile_generate_finish (SysprofProfile     *profile,
-                                      GAsyncResult  *result,
-                                      GError       **error)
+sysprof_callgraph_profile_generate_finish (SysprofProfile  *profile,
+                                           GAsyncResult    *result,
+                                           GError         **error)
 {
   SysprofCallgraphProfile *self = (SysprofCallgraphProfile *)profile;
   StackStash *stash;
